"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utility_1 = require("../common/utility");
const module_1 = require("../code-object/module");
/**
 * # Transcriber
 * We always write code in one language and translate(compile) it to another, however, in both language, the code we write directly represents the execution flow.
 * When we write draft, we don't expect it executes as is, instead, it's designed to be transformed, not executed.
 * That's why we choose another word to differentiate it from compiler.
 */
class Transcriber {
    constructor(_module) {
        this.m_Module = new module_1.ModuleCode(_module);
        this.m_ClassMap = new Map();
        this.m_MethodMap = new Map();
        this.m_ContextMap = new Map();
        this.m_InlineContextMap = new Map();
        this.m_DSLMap = new Map();
        this.m_Plugins = new Array();
    }
    /**
     * A transcriber is just a container, it's a collection of plugins.
     * For example, to transform local context, we have `draft-plugin-local-context`, to add class method to class, we have `draft-plugin-class`, and `draft-plugin-dsl` for DSL extension support.
     */
    Transcribe() {
        this.m_Plugins.forEach(plugin => plugin.Transcribe());
        return utility_1.ToString(this.m_Module.m_File, {
            comments: false,
        });
    }
    AddDSL(name, dsl) {
        this.m_DSLMap.set(name, dsl);
    }
    GetDSL(name) {
        return this.m_DSLMap.get(name);
    }
    GetLocalContext(name) {
        return this.m_ContextMap.get(name);
    }
    TraverseLocalContext(callback) {
        this.m_ContextMap.forEach((context, name) => callback(context, name));
    }
    TraverseInlineContext(callback) {
        this.m_InlineContextMap.forEach(context => callback(context));
    }
    GetClass(name) {
        return this.m_ClassMap.get(name);
    }
    TraverseMethod(callback) {
        this.m_MethodMap.forEach((methods, class_name) => callback(methods, class_name));
    }
}
exports.Transcriber = Transcriber;
